<?php

abstract class Abstract_Expapay_Responsive {
	
	const STATUS_PENDING = 'PENDING';
	const STATUS_FORM_SHOWN = 'FORM_SHOWN';
	const STATUS_ENROLLED = 'ENROLLED';
	const STATUS_TDS_CHECKING = 'TDS_CHECKING';
	const STATUS_RESERVED = 'RESERVED';
	const STATUS_CAPTURED = 'CAPTURED';
	const STATUS_PARTIAL = 'PARTIAL';
	const STATUS_REFUNDED = 'REFUNDED';
	const STATUS_COMPLETE = 'COMPLETE';
	const STATUS_REJECTED = 'REJECTED';
	const STATUS_FAILED = 'FAILED';
	
	protected $status;
	
	public function __construct( $data ) {
		
		if ( is_array( $data ) ) {
			foreach ( $data as $key => $val ) {
				$this->set_property( $key, $val );
			}
		}
	}
	
	public function get_status_label() {
		switch ( $this->status ) {
			case self::STATUS_PENDING:
				return 'Новый платёж. Ожидается указание способа оплаты и платёжных данных.';
			case self::STATUS_FORM_SHOWN:
				return 'Плательщик направлен на платёжную страницу.';
			case self::STATUS_ENROLLED:
				return 'Данные банковской карты проверены; платёж ожидает прохождение 3DSecure-авторизации.';
			case self::STATUS_TDS_CHECKING:
				return 'Плательщик направлен на страницу 3DSecure-авторизации.';
			case self::STATUS_RESERVED:
				return 'Средства успешно зарезервированы на карте плательщика.';
			case self::STATUS_CAPTURED:
				return 'Средства успешно списаны со счёта плательщика.';
			case self::STATUS_PARTIAL:
				return 'Часть суммы по счёту была возвращена плательщику.';
			case self::STATUS_REFUNDED:
				return 'Вся сумма по счёту была возвращена плательщику.';
			case self::STATUS_COMPLETE:
				return 'Средства переведены на счёт получателя.';
			case self::STATUS_REJECTED:
				return 'Операция отклонена банком или платёжным шлюзом.';
			case self::STATUS_FAILED:
				return 'При выполнении последней операции произошла ошибка.';
			default:
				return '';
		}
	}
	
	private function set_property( $property, $val ) {
		
		if ( property_exists( $this, $property ) ) {
			if ( $property == 'error' ) {
				$val = $this->json_decode( $val );
			}
			$this->$property = $val;
		}
	}
	
	private function json_decode( $s ) {
		$s = stripslashes( $s );
		$s = str_replace( '\'', '"', $s );
		$s = preg_replace( '/(\d+)/i', '"\1"', $s );
		return (array)json_decode( $s );
	}
	
	/**
	 * @return mixed
	 */
	public function get_status() {
		return $this->status;
	}
}
