<?php

require_once plugin_dir_path( dirname( __FILE__ ) ) . '/components/expapay-gateway/class-expapay.php';

/**
 *
 * @since      1.0.0
 * @package    WC_Expapay_Payment_Gateway
 * @subpackage WC_Expapay_Payment_Gateway/woocommerce
 */
class WC_Expapay_Payment_Gateway_Main extends WC_Payment_Gateway {
	
	private $account_id;
	private $signing_key;
	private $redirect_page_id;
	private $title_submit_order_page;
	private $name_input_payment = 'expapay-payment';
	private $test_mode = 'expapay-payment';
	private $expapay;
	
	public function __construct() {
		
		$this->id = 'expapay'; // payment gateway plugin ID
		// URL of the icon that will be displayed on checkout page near your gateway name
		//$this->icon               = WP_PLUGIN_URL . "/" . plugin_basename( dirname( __FILE__ ) ) . '/img/logo.svg';
		$this->has_fields         = false; // in case you need a custom credit card form
		$this->method_title       = 'Expapay Gateway';
		$this->method_description = $this->get_custom_description(); // will be displayed on the options page
		// gateways can support subscriptions, refunds, saved payment methods,
		$this->supports = array(
			'products',
			//'refunds',
			//'pre-orders'
		);
		
		// Method with all the options fields
		$this->init_form_fields();
		
		// Load the settings.
		$this->init_settings();
		$this->title                   = $this->get_option( 'title' );
		$this->description             = $this->get_option( 'description' );
		$this->enabled                 = $this->get_option( 'enabled' );
		$this->title_submit_order_page = $this->get_option( 'title_submit_order_page' );
		//$this->testmode        = 'yes' === $this->get_option( 'testmode' );
		$this->account_id       = $this->get_option( 'account_id' ); // account id
		$this->signing_key      = $this->get_option( 'signing_key' ); // signing key
		$this->redirect_page_id = $this->get_option( 'redirect_page_id' );
		$this->test_mode        = $this->get_option( 'test_mode' );
		
		
		// This action hook saves the settings
		
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id,
			array( $this, 'process_admin_options' ) );
		
		// We need custom JavaScript to obtain a token
		add_action( 'wp_enqueue_scripts', array( $this, 'payment_scripts' ) );
		
		// You can also register a webhook here
		add_action( 'woocommerce_api_' . $this->id, array( $this, 'webhook' ) );
		
		add_action( 'woocommerce_receipt_expapay', array( &$this, 'order_page' ) );
		
	}
	
	
	/**
	 * Plugin options
	 */
	public function init_form_fields() {
		
		$this->form_fields = array(
			
			'enabled' => array(
				'title'       => 'Включить/Выключить',
				'label'       => 'Включить Expapay Gateway',
				'type'        => 'checkbox',
				'description' => 'Показать в списке платежей как способ оплаты.',
				'default'     => true
			),
			
			'test_mode' => array(
				'title'   => 'Тестовый режим',
				'label'   => 'Включить тестовый режим',
				'type'    => 'checkbox',
				'default' => 'no',
			),
			
			'title'                   => array(
				'title'       => 'Заголовок',
				'type'        => 'text',
				'description' => 'Заголовок, который пользователь видит во время оформления заказа.',
				'default'     => 'Онлайн платежи Expapay',
				'desc_tip'    => true,
			),
			'description'             => array(
				'title'       => 'Описание',
				'type'        => 'textarea',
				'description' => 'Описание, которое пользователь видит во время оформления заказа.',
				'default'     => 'Оплата банковской картой.',
			),
			'title_submit_order_page' => array(
				'title'       => 'Название кнопки оплаты',
				'type'        => 'text',
				'description' => 'Название кнопки, на странице оплаты заказа.',
				'default'     => 'Оплатить',
				'desc_tip'    => true,
			),
			'account_id'              => array(
				'title'       => 'Идентификатор аккаунта',
				'description' => ' Значение можно посмотреть в личном кабинете Expapay.',
				'type'        => 'text'
			),
			'signing_key'             => array(
				'title'       => 'Ключ для подписи',
				'description' => ' Значение можно посмотреть в личном кабинете Expapay.',
				'type'        => 'password',
			),
			'redirect_page_id'        => array(
				'title'    => 'Страница ответа',
				'type'     => 'select',
				'options'  => $this->get_all_pages( 'По умолчанию' ),
				'desc_tip' => true
			),
		
		);
	}
	
	/**
	 * You will need it if you want your custom credit card form
	 */
	public function payment_fields() {
		if ( $this->description ) {
			echo wpautop( wptexturize( $this->description ) );
		}
	}
	
	/**
	 * Custom CSS and JS, in most cases required only when you decided to go with a custom credit card form
	 */
	public function payment_scripts() {
		
		
	}
	
	/**
	 * Fields validation
	 */
	public function validate_fields() {
		
		
	}
	
	
	/**
	 * We're processing the payments here
	 */
	public function process_payment( $order_id ) {
		
		$order = new WC_Order( $order_id );
		
		if ( version_compare( WOOCOMMERCE_VERSION, '2.1.0', '>=' ) ) {
			/* 2.1.0 */
			$checkout_payment_url = $order->get_checkout_payment_url( true );
		} else {
			/* 2.0.0 */
			$checkout_payment_url = get_permalink( get_option( 'woocommerce_pay_page_id' ) );
		}
		
		return array(
			'result'   => 'success',
			'redirect' => add_query_arg( 'order_pay', $order_id, $checkout_payment_url )
		);
		
	}
	
	/**
	 * In case you need a webhook, like PayPal IPN etc
	 */
	public function webhook() {
		
		global $woocommerce;
		
		$notification = new Expapay_Notification( $_POST );
		$order        = wc_get_order( $notification->get_order_id() );
		
		if ( $notification->is_validation( $this->get_option( 'signing_key' ) ) ) {
			
			if ( $order instanceof WC_Order ) {
				$order->payment_complete();
				$order->add_order_note( $note = 'Заказ оплачен Expapay' );
				if ( $this->is_redirect_page_id() ) {
					wc_add_notice( $note );
					$woocommerce->cart->empty_cart();
				}
			} else {
				wc_add_notice( 'Ошибка. Заказ не найден', 'error' );
			}
			
		} else {
			if ( $order instanceof WC_Order ) {
				$order->update_status( 'failed' );
				$order->add_order_note( 'ERROR: ' . $notification->get_error_mess() );
			} elseif ( $this->is_redirect_page_id() ) {
				wc_add_notice( $notification->get_error_mess(), 'error' );
			}
		}
		wp_redirect( $this->get_redirect_url( $order ) );
	}
	
	private function is_redirect_page_id() {
		return $this->redirect_page_id != '' && $this->redirect_page_id != 0 ? true : false;
	}
	
	private function get_redirect_url( $order ) {
		
		if ( ! $this->is_redirect_page_id() ) {
			return $order instanceof WC_Order ? $order->get_checkout_order_received_url() : '/';
		}
		
		return get_permalink( $this->redirect_page_id );
	}
	
	private function webhook_url() {
		//For wooCoomerce 2.0
		return add_query_arg( 'wc-api', $this->id, get_site_url() . '/' );
	}
	
	/**
	 * Order page
	 *
	 * @param $order_id
	 */
	public function order_page( $order_id ) {
		
		if ( @$_POST[ $this->name_input_payment ] ) {
			$this->send_purchase( $order_id );
		} else {
			require plugin_dir_path( dirname( __FILE__ ) ) . 'woocommerce/partials/order-page-form.php';
		}
	}
	
	
	public function get_name_input_payment() {
		return $this->name_input_payment;
	}
	
	private function send_purchase( $order_id ) {
		
		$this->set_expapay( $order_id );
		
		$responsive = $this->expapay->get_responsive();
		if ( $responsive instanceof Expapay_Purchase_Responsive ) {
			if ( $responsive->get_error() ) {
				$this->add_error_notice( $responsive->get_error() );
			} else {
				wp_redirect( $responsive->get_redirect_to() );
				exit;
			}
		} else {
			$this->add_error_notice( 'Ошибка' );
		}
		
		$this->refresh();
	}
	
	private function set_expapay( $order_id ) {
		$order         = wc_get_order( $order_id );
		$this->expapay = new Expapay( $this->is_test_mode() );
		$this->expapay->request_purchase(
			$this->account_id,
			$order->get_total(),
			$order->get_id(),
			$this->signing_key );
	}
	
	private function refresh() {
		wp_redirect( $_SERVER['HTTP_REFERER'] );
	}
	
	private function add_error_notice( $mess ) {
		wc_add_notice( $mess, 'error' );
	}
	
	// get all pages
	private function get_all_pages( $title = false, $indent = true ) {
		$wp_pages  = get_pages( 'sort_column=menu_order' );
		$page_list = array();
		if ( $title ) {
			$page_list[] = $title;
		}
		foreach ( $wp_pages as $page ) {
			$prefix = '';
			// show indented child pages?
			if ( $indent ) {
				$has_parent = $page->post_parent;
				while ( $has_parent ) {
					$prefix     .= ' - ';
					$next_page  = get_post( $has_parent );
					$has_parent = $next_page->post_parent;
				}
			}
			// add to page list array array
			$page_list[ $page->ID ] = $prefix . $page->post_title;
		}
		
		return $page_list;
	}
	
	private function is_test_mode() {
		return $this->test_mode == 'yes' ? true : false;
	}
	
	private function get_custom_description() {
		return 'Установить в личном кабинете Expapay. Return path: <b>' . $this->webhook_url() . '</b>';
	}
}
